
#ifndef _SEG_REGION_H_
#define _SEG_REGION_H_

#include "Vector3.h"

#include <string>
#include <vector>
#include <stack>
#include <map>
using namespace std;



class RegionNode
{
public:
	int rfile;
	int rid;
	vector < RegionNode* > cnodes;
	
	void WriteToFile ( FILE *fp );
};



class Region
{
public:

	// list of voxels, storing i,j,k coordinates
	vector< Vector3f > vi;

	// center of mass, bounding rad, etc.
	Vector3f COM;
	double dBoundRad, maxD;
	
	Region *p_reg;
	vector < Region* > sub_regs;

	RegionNode *node;

	// a map of voxels facilitating efficient overlap-testing
	map < int, map< int, map< int, int> > > mi;

	Region ()
	{
		COM = Vector3f (0,0,0);
		dBoundRad = 0.0;
		maxD = -1e100;
		node = NULL;
		p_reg = NULL;
	}

	int Overlap ( Region &r );
	int OverlapF ( Region &r );

	void MakeVerticesMap ();
	void COM_and_bRad ();
};


class RegsConnection
{
public:
	float N, totD, minD, maxD;

	RegsConnection ( /*int pr1, int pr2*/ ) {
		//r1 = pr1;
		//r2 = pr2;
		N = 0.0;
		totD = 0.0;
		minD = 1e9;
		maxD = -1e9;
	}
};


class Regions
{
public:
	map < int, Region >								mRegs;
	map < int, map < int, RegsConnection* > >		mCons;
	string name;

	Regions () {}
	~Regions () { Clear(); }

	void Clear ();
	void Write ( const char *fname );
	void Read ( const char *fname );
	void Compute_COMs_bRads ();
	Regions* GroupByOverlap ( Regions &sregs );

	void MakeNodes ( int rfile );
	void AssignCNodesToSubRegs ();
	void AssignNodesByOverlap ( Regions &prev_regs, Regions &sharpened_regs );
};




#endif //_SEG_REGION_H_

